#! /usr/bin/perl -w

#
# This script updates grub configuration and initrd images after the
# (un)installation of MicroSoft integration components. It is invoked
# in the post-(un)install scriptlet of the mshvic RPM.
#

use strict;
use diagnostics;
use POSIX;

sub Usage();
sub Clean(@);
sub CreateInitrd($$);
sub UpdateGrub($$);

my $debug = 0;
print STDOUT " >>> $0 @ARGV\n" if $debug;

Usage() unless (scalar(@ARGV) == 1 or scalar(@ARGV) == 2);

print STDOUT "\@ARGV @ARGV\n" if $debug;

my $action = $ARGV[0];
Usage unless grep {$_ eq $action} qw(install uninstall update clean);

my $kernelvariant = undef;
if (defined $ARGV[1]){
    $kernelvariant = $ARGV[1];
    Usage unless grep {$_ eq $kernelvariant} qw(kernel kernel-PAE);
}

# Check the directory that stores the state files

my $statedir = "/var/lib/mshvic";
if (not -d $statedir){
    print STDERR "No directory $statedir found, exiting\n";
    exit 1;
}

# Get installed kernels

my @kernels_to_process = my @all_kernels = ();

opendir(BOOT,"/boot") || die "can't opendir /boot: $!";
for my $kernel (sort map {s/^vmlinuz-//;$_} grep {/^vmlinuz-(\S+)/} readdir(BOOT)){
    push(@all_kernels,$kernel);
    if (defined $kernelvariant){
        if ($kernel =~ /PAE$/){
            next unless $kernelvariant eq "kernel-PAE";
        }else{
            next unless $kernelvariant eq "kernel";
        }
    }
    push(@kernels_to_process,$kernel);
}
closedir BOOT;
print STDOUT "\@all_kernels        = @all_kernels\n" if $debug;
print STDOUT "\@kernels_to_process = @kernels_to_process\n" if $debug;

# Housekeeping

Clean(@all_kernels);
exit 0 if $action eq "clean";

# Handle them

my $err = 0;
for my $kernel (@kernels_to_process){
    print STDOUT "Processing kernel $kernel\n" if $debug;
    
    # Check state file
    if ($action eq "install"){
        if (-f "$statedir/$kernel"){
            print STDOUT "Nothing to do!\n" if $debug;
            next;
        }
    }elsif ($action eq "uninstall"){
        unless (-f "$statedir/$kernel"){
            print STDOUT "Nothing to do!\n" if $debug;
            next;
        }
    }elsif ($action eq "update"){
        # just do it :)
    }
    
    # Recreate initrd
    if (CreateInitrd($kernel,$action)){
        $err++;
        next;
    }
    
    # Update grub configuration
    if (UpdateGrub($kernel,$action)){
        CreateInitrd($kernel,($action eq "uninstall" ? "install" : "uninstall")); # roll back
        $err++;
        next;
    }
    
    # Create a state file
    if ($action eq "install" or $action eq "update"){
        print STDOUT "  Create file \"$statedir/$kernel\"\n" if $debug;
        system("/bin/touch $statedir/$kernel") && print STDERR "Could not create file \"$statedir/$kernel\"\n";
    }else{
        print STDOUT "  Remove file \"$statedir/$kernel\"\n" if $debug;
        unlink "$statedir/$kernel";
    }
}

# Done

exit $err;

#
# Recreate initrd
#
sub CreateInitrd($$){
    my ($kernel,$action) = @_;

    my $command = "/sbin/mkinitrd -f /boot/initrd-$kernel.img $kernel";
    $command .= " --with=vmbus --with=blkvsc --with=storvsc --with=netvsc" if $action eq "install" or $action eq "update";

    print STDOUT "  Now run \"$command\"\n" if $debug;
    system($command) || return 0;

    print STDOUT "  Could not run $command: $!\n" if $debug;
    return 1;
}

#
# Update GRUB configuration
#
sub UpdateGrub($$){
    my ($kernel,$action) = @_;

    my $arch = (POSIX::uname())[4];
    my $timekeeping = ($arch eq "x86_64" ? "divider=10" : "divider=10 clocksource=acpi_pm"); # improve timekeeping as well...

    my $command = "/sbin/grubby --update-kernel=/boot/vmlinuz-$kernel ";
    $command .= ($action eq "uninstall" ? "--remove-args=" : "--args=") . "\"hda=noprobe hdb=noprobe $timekeeping\"";

    print STDOUT "  Now run \"$command\"\n" if $debug;
    system($command) || return 0;

    print STDOUT "  Could not run $command: $!\n" if $debug;
    return 1;

}

#
# Housekeeping: remove state file for kernels not found on the system
#
sub Clean(@){
    my @kernels = @_;
    print STDOUT "Housekeeping\n  \@kernels     = @kernels\n  \@all_kernels = @all_kernels\n" if $debug;
    if (opendir(STATE,$statedir)){
        for my $file (grep { ! /^\./ && -f "$statedir/$_" } readdir(STATE)){
            if (not grep {$file eq $_} @all_kernels){
                unlink "$statedir/$file";
                print STDOUT "  Remove file \"$statedir/$file\"\n" if $debug;
            }
        }
        closedir STATE;
    }
    return;
}

#
# Print usage and exit
#
sub Usage(){
    print STDERR "Usage: $0 [ install | uninstall | update| clean ] [ kernel | kernel-PAE ]\n";
    exit 1;
}

__END__
